#include <stdio.h>
#include <limits.h>
#include <string.h>

#define DUMPBUFSIZE 512

int read32(unsigned char * buf) {
	return (buf[0]<<24)|(buf[1]<<16)|(buf[2]<<8)|buf[3];
}

int dump(FILE * infile, FILE * outfile, int offset, int size) {
	char dumpbuf[DUMPBUFSIZE];
	int oldoff;
	oldoff=ftell(infile);

	fseek(infile,offset,SEEK_SET);

	for (;size>=DUMPBUFSIZE;size-=DUMPBUFSIZE) {
		fread(dumpbuf,1,DUMPBUFSIZE,infile);
		fwrite(dumpbuf,1,DUMPBUFSIZE,outfile);
	}
	if (size) {
		fread(dumpbuf,1,size,infile);
		fwrite(dumpbuf,1,size,outfile);
	}
	fseek(infile,oldoff,SEEK_SET);
}

int main(int argc, char ** argv) {
	FILE * infile;
	unsigned char buf[4];
	int thesunshines=1;

	/* read .baa and dump .bst and .bstn */

	if (argc!=2) {
		fprintf(stderr,"arg!\n");
		return 1;
	}
	infile=fopen(argv[1],"rb");
	if (!infile) {
		fprintf(stderr,"fail open %s\n",argv[1]);
		return 1;
	}

	fread(buf,1,4,infile);
	if (memcmp(buf,"AA_<",4)) {
		fprintf(stderr,"format not recognized\n");
		return 1;
	}

	while (thesunshines) {
		/* read chunk name */
		fread(buf,1,4,infile);
		if (!memcmp(buf,"bst ",4)) {
			int bstoffset;
			int bstnoffset;
			char filename[PATH_MAX+1];
			FILE * outfile;
			memset(filename,0,PATH_MAX+1);

			printf("\"bst \"\t");
			fread(buf,1,4,infile);
			bstoffset=read32(buf);
			printf("%08x\t",read32(buf));

			fread(buf,1,4,infile);
			bstnoffset=read32(buf);
			printf("%08x\n",read32(buf));

			snprintf(filename,PATH_MAX,"%s.bst",argv[1]);
			outfile=fopen(filename,"wb");
			if (!outfile) {
				fprintf(stderr,"couldn't open %s for writing\n",filename);
				return 1;
			}
			if (dump(infile,outfile,bstoffset,bstnoffset-bstoffset)) {
				fprintf(stderr,"error dumping to %s\n");
				return 1;
			}
			fclose(outfile);
		} else if (!memcmp(buf,"bstn",4)) {
			int bstnoffset;
			int bstnendoffset;
			char filename[PATH_MAX+1];
			FILE * outfile;
			memset(filename,0,PATH_MAX+1);

			printf("\"bstn\"\t");
			fread(buf,1,4,infile);
			bstnoffset=read32(buf);
			printf("%08x\t",read32(buf));

			fread(buf,1,4,infile);
			bstnendoffset=read32(buf);
			printf("%08x\n",read32(buf));

			snprintf(filename,PATH_MAX,"%s.bstn",argv[1]);
			outfile=fopen(filename,"wb");
			if (!outfile) {
				fprintf(stderr,"couldn't open %s for writing\n",filename);
				return 1;
			}
			if (dump(infile,outfile,bstnoffset,bstnendoffset-bstnoffset)) {
				fprintf(stderr,"error dumping to %s\n");
				return 1;
			}
			fclose(outfile);
		} else if (!memcmp(buf,"ws  ",4)) {
			printf("\"ws  \"\t");
			fread(buf,1,4,infile);
			printf("%08x\t",read32(buf));
			fread(buf,1,4,infile);
			printf("%08x\t",read32(buf));
			fread(buf,1,4,infile);
			printf("%08x\n",read32(buf));
		} else if (!memcmp(buf,"bnk ",4)) {
			printf("\"bnk \"\t");
			fread(buf,1,4,infile);
			printf("%08x\t",read32(buf));
			fread(buf,1,4,infile);
			printf("%08x\n",read32(buf));
		} else if (!memcmp(buf,"bsc ",4)) {
			printf("\"bsc \"\t");
			fread(buf,1,4,infile);
			printf("%08x\t",read32(buf));
			fread(buf,1,4,infile);
			printf("%08x\n",read32(buf));
		} else if (!memcmp(buf,"bfca",4)) {
			printf("\"bfca\"\t");
			fread(buf,1,4,infile);
			printf("%08x\n",read32(buf));
		} else if (!memcmp(buf,">_AA",4)) {
			/* haymaking time is over */
			thesunshines=0;
			printf("\">_AA\"\n");
		} else {
			fprintf(stderr,"unrecognized chunk, %c%c%c%c (%08x)\n",
					buf[0],buf[1],buf[2],buf[3],read32(buf));
			return 1;
		}
	}

	return 0;
}
